
# q-expansion of newform 5824.2.a.ck, downloaded from the LMFDB on 29 April 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 5824
weight = 2
poly_data = [18, 40, -2, -13, 0, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0, 0], 1], [[0, 1, 0, 0, 0], 1], [[-5, 0, 1, 0, 0], 1], [[-6, -7, 1, 1, 0], 1], [[5, -7, -8, 1, 1], 1]]

hecke_ring_character_values = None
aps_data = [[0, 0, 0, 0, 0], [0, 1, 0, 0, 0], [-1, 0, 0, 1, 0], [1, 0, 0, 0, 0], [2, 0, 0, 1, 0], [-1, 0, 0, 0, 0], [1, 0, 1, -1, 0], [-1, 0, 2, -1, 0], [0, -1, 0, 0, -1], [-1, 0, 1, 1, 1], [2, -2, 1, 1, 0], [-3, 1, 0, -1, -1], [5, 1, -1, -1, 0], [0, 0, 1, 0, 0], [-1, -2, -1, 0, -1], [-2, 2, 1, 0, 0], [3, 0, 0, -1, 1], [2, -2, 1, 1, -1], [0, 2, 1, 1, 1], [2, -2, 1, 0, -1], [1, -1, -3, 1, -1], [-1, 1, -2, 1, 0], [2, 0, -2, 2, -1], [2, 0, 2, 0, -1], [5, 1, 1, 1, -1], [-1, 0, 0, 0, -1], [-2, 0, 2, 0, 2], [1, -2, 2, -3, -1], [-4, 0, -1, -2, -1], [-2, 4, 0, -1, 1], [2, 3, -1, -2, -1], [8, -4, -2, 0, 0], [2, -2, 3, -2, -1], [-5, -2, -1, 3, 0], [-3, 1, 0, -1, -1], [-6, 0, -1, -2, -1], [-3, 0, 0, 0, 1], [2, -4, 0, -2, 0], [5, 6, -2, 1, 0], [-5, -2, 1, -3, 2], [-2, 0, 1, 0, 2], [1, 4, -4, 2, 1], [7, 0, 0, -3, -1], [0, -4, 0, 0, -2], [-9, -1, -3, 3, -2], [5, -2, 3, 1, -2], [3, 0, 3, 3, 1], [8, 6, -3, 1, 0], [12, -2, 0, -6, 0], [7, -4, 2, 1, 1], [3, -8, 0, 0, 0], [10, 4, -2, 0, 2], [5, 0, 0, 1, 2], [2, 1, 0, -6, 0], [11, 4, -7, 1, -2], [2, 4, 7, -2, 0], [-2, 2, 1, 1, -1], [12, 4, 3, -1, 1], [9, 4, -5, 1, -1], [-12, -2, 1, -2, 1], [3, -9, 3, -1, -2], [6, 0, 2, 0, 3], [-2, 4, -2, 6, -1], [-5, -6, -5, 5, 0], [3, 0, -1, -1, -2], [-3, 3, -1, 5, 0], [4, 6, -5, 5, 3], [8, 0, -2, 5, 1], [4, -4, 0, 0, 0], [-6, -4, 2, -4, 1], [8, 1, -5, -4, -3], [8, -2, -1, 6, 3], [-11, -6, 1, 1, 0], [1, 2, 2, -3, -3], [14, 2, -3, -4, -1], [20, 0, -3, -1, -1], [3, -6, 4, -5, -3], [-4, -4, -2, -4, -1], [-8, -8, 2, 0, 0], [-6, -12, 4, 0, -3], [4, 5, 2, 0, 0], [-11, -1, 4, -1, 3], [18, -2, 2, -2, 0], [-2, 0, 10, 0, 2], [4, -4, -2, 2, -2], [-5, -8, 3, -3, 1], [-22, 6, 2, 6, 2], [-4, -2, 5, -4, 1], [12, -4, 2, 0, 4], [-26, 0, -1, 2, 1], [-11, 6, 1, 1, 2], [10, 2, 6, 0, -1], [-6, 2, 4, 2, 2], [3, 2, -6, 3, -3], [6, 4, -2, 3, 0], [-8, 6, 2, 2, 0], [2, 4, -4, 0, 1], [-4, -10, 2, 0, -4], [-13, -1, 3, 7, 0], [-18, 0, 3, 0, 1], [5, 2, -3, -3, 3], [-1, -5, -1, -5, 2], [3, -9, -3, -1, 0], [-11, -4, -2, 8, 0], [-11, 6, -1, 7, -1], [10, 2, 2, -2, 2], [-4, 6, -8, -8, -1], [-7, 4, 8, 1, 4], [3, 5, 0, 9, -2], [-12, -6, 0, -4, 2], [7, 0, -5, 7, 0], [3, 1, 1, 7, 0], [-2, -4, 2, 0, 0], [-16, -2, -6, 8, -2], [-16, 2, -4, 0, 2], [-13, 0, -4, 4, 2], [16, -2, 4, -2, 0], [-25, 0, -3, -3, 2], [4, -8, 0, -4, -2], [9, 0, 5, 5, 3], [-1, -2, 0, -1, 2], [8, 0, 2, -1, -3], [17, 4, -4, 2, 1], [12, -2, -1, -7, 1], [1, -8, 6, -1, -4], [13, -2, -1, 5, 3], [1, 3, -2, 3, -5], [13, 2, -5, -5, -2], [-14, -6, 4, 2, 2], [-15, 2, 2, -3, 2], [14, -2, -3, 0, 1], [-2, 6, 4, 0, -2], [-13, 3, -6, 5, 4], [-1, -4, 7, -1, -3], [-10, -1, -5, 8, -2], [8, 7, -1, 2, 0], [14, -2, 4, -2, 0], [0, -2, -2, 6, -1], [12, -6, 3, 1, -5], [-9, -12, 7, 3, -1], [-2, 0, 6, 6, 0], [-6, 2, 2, -6, -2], [-6, 7, -1, -10, -1], [14, 4, 0, 2, 0], [-13, 6, -1, 5, 0], [-6, 4, 1, -7, -3], [-43, 2, 0, 1, 2], [-18, 6, 6, -2, 0], [-1, -3, 1, 3, 4], [4, 2, 2, -6, 0], [-23, -5, -2, 3, -1], [-4, -2, 2, 0, 4], [1, 0, -6, -11, -1], [2, 0, -4, -8, 2], [-11, -8, 7, 1, 3], [16, 4, 9, -4, 4], [-15, 1, 1, 3, 0], [-12, -9, 1, 2, 2], [-13, 8, -5, 1, -2], [3, -6, 10, 1, 0], [-6, 0, -12, 6, 0], [9, 4, -3, -7, -1], [-18, -8, 7, -4, 3], [-13, 1, 3, 3, 2], [-6, 12, 7, -4, -1], [25, -6, 6, 1, 0], [8, -3, -7, 12, 3], [-14, 14, -3, 1, 3], [26, 4, 9, -6, 3], [-5, 2, 7, -3, 2], [28, -6, -5, -6, -5], [2, 12, 6, -4, -2], [24, -1, -2, 0, -1], [10, 6, -1, -8, 1], [-4, -8, 8, -2, 0], [-18, 4, -4, -6, -6], [12, 6, -1, -1, -1], [-14, -6, -2, -6, 2], [11, 2, -7, 3, -6], [25, 4, -6, -2, -1], [2, -12, 6, 4, -2], [6, 3, -4, -14, -2], [3, 2, -10, 1, -1], [17, 9, -9, 5, 3], [-12, -2, -3, 6, 3], [17, 14, -1, -1, 0], [-6, -12, 5, -6, 0], [-18, 0, -8, -6, -1], [6, -4, -2, 10, 4], [7, 2, -5, 0, 5], [13, -4, -3, -13, -2], [-10, -2, 11, -5, 3], [23, -2, 1, -3, -5], [-27, 0, -6, 5, 0], [15, 4, 13, -5, 1], [5, -2, -1, 15, 4], [14, -2, -1, 8, 3], [-27, -16, 3, 1, -1], [-23, 12, 2, 5, 4], [20, -10, 0, 2, 4], [37, -1, -4, -1, 1], [-17, 2, 1, 3, 4], [-4, 12, -2, -8, -1], [-10, -4, 2, 0, 2], [13, 8, 0, -3, -2], [28, 0, 4, 6, 2], [2, 2, 8, -4, 9], [-12, -4, -1, 6, -2], [-24, 8, 1, 10, 5], [-23, 7, -9, -3, 2], [-1, 0, 4, -14, -4], [9, 0, 2, 0, -1], [-19, 1, 7, 3, 2], [18, 4, 6, 2, -4], [0, 2, 9, 5, 0], [22, -9, 1, -12, 0], [-5, -17, 2, 5, -4], [14, -12, 6, -6, -4], [4, -2, 0, -8, -4], [21, -1, 10, -5, 1], [14, -20, -7, 4, 2], [21, -8, -3, -12, -2], [6, -6, 0, -8, 0], [-4, -2, -4, 0, 6], [-23, 1, 7, -7, -4], [-10, 8, -3, 6, 0], [-14, -18, 8, 4, 0], [-29, -15, 10, -7, 2], [13, 0, 3, -5, 4], [24, 4, -4, 1, 4], [-31, -2, -6, 1, -3], [-19, 2, 3, -11, -2], [-6, -2, -2, 4, -6], [5, 8, -1, -6, 4], [1, 5, -1, 9, -8], [1, -10, -9, 2, 1], [-11, 7, 3, -5, 0], [-35, -1, 5, 1, 0], [11, 0, 3, 5, -1], [4, -10, 0, 6, -2], [-20, -10, 9, 7, 1], [-25, 9, -5, -1, -2], [-38, -2, 8, -4, -2], [-14, 10, -6, 14, 6], [-4, 7, -7, 14, -4], [7, -10, 5, -3, 2], [21, -2, 5, 2, 3], [-14, 18, -2, -10, -3], [-2, -4, 0, 5, 0], [-10, -7, 4, -10, -5], [-13, 5, -5, -13, -4], [24, -10, -2, 8, 4], [-12, 6, 4, 4, 4], [8, -4, 2, -10, -2], [12, 2, -5, 9, -1], [-14, 0, 14, -2, 4], [-3, 2, -5, -1, -3], [28, 2, 12, -12, 0], [-28, -2, 5, 3, 1], [32, -7, -5, -6, -2], [25, 12, -7, -9, -1], [0, -2, -1, 4, -4], [-2, 0, -6, 0, 1], [-11, -12, 0, 0, 3], [-3, 13, 7, -1, 0], [-10, -4, 2, 2, 6], [7, -3, -1, -1, 6], [17, -9, 5, -15, -5], [-8, -16, 4, 9, -4], [-9, -18, -2, 3, -5], [5, -12, -6, -4, 5], [12, 4, -2, -15, -4], [19, -7, 5, -15, -4], [-17, 0, 3, 5, -3], [-23, 8, -11, 9, 6], [22, -8, -4, -14, -6], [-24, -6, -9, 6, -1], [-29, 5, 6, 9, 5], [-8, -18, 10, -6, -4], [8, -8, 4, 16, 2], [-32, 6, -6, 4, -2], [-6, -6, 0, -14, 4], [16, 10, 6, 0, 4], [2, -6, 2, 2, -3], [7, 16, -2, -3, -1], [37, -1, 2, -9, -2], [-3, 0, 4, 12, 2], [-18, 14, 8, 6, 4], [-14, 12, 3, 9, 7], [1, 8, 13, 3, 2], [8, -8, 7, -10, 0], [21, 19, -9, 1, 0], [0, 4, -12, -2, -2], [-17, -14, 4, 1, -6], [30, -2, 7, 2, 0], [10, -8, -6, 8, 4], [38, -6, 4, 0, 2], [14, -10, -3, -13, -5], [0, 7, -10, -12, -4], [8, 14, 8, 0, 1], [17, -10, 9, 7, 0], [-3, 17, 12, -1, 3], [25, 0, 7, -5, 5], [-21, 12, 9, 5, 3], [17, 4, -1, -1, -3], [22, 0, -1, 2, -3], [-1, -2, 2, 1, -3], [9, -2, 5, -13, 6], [16, -4, -4, 6, 2], [-6, 22, 1, 4, 0], [-3, 2, 5, -13, 0], [-14, 8, 7, 4, 3], [-35, -4, 9, -4, -2], [-54, 14, 3, 6, 2], [-10, 8, -5, 4, 3], [-14, -6, -12, 8, -2], [14, -10, 4, 4, 7], [12, 2, 7, 12, -3], [1, 0, 10, -3, 2], [-5, 4, -10, 10, -4], [21, -11, 5, -11, 2], [-4, -12, 4, 12, 2], [-53, 4, -6, 6, 1], [16, -16, 0, -10, -4], [-26, 6, -10, -2, 4], [7, -10, 1, -7, 8], [42, 2, -1, -9, 1], [4, -6, -4, -6, -1], [5, 0, 8, -9, 1], [13, -12, 4, 1, 3], [-33, 7, 8, 5, 1], [28, -2, -9, 10, 0], [9, 2, -3, -1, 6], [-19, 6, -4, 5, -2], [0, -12, -16, 5, 0], [21, 6, -2, -5, 2], [8, 4, -9, -14, 1], [7, 11, -17, 13, 2], [31, -2, 8, 13, -1], [28, 18, -6, -8, 0], [-24, 4, -11, 6, -1], [-5, 4, -1, -14, -6], [25, -4, -4, 5, 5], [-10, -15, 5, -10, 5], [-17, -13, -8, 5, -3], [-13, -22, 2, 3, 1], [25, -2, 7, -5, -4], [22, 2, -5, 0, 5], [-9, -6, 3, 10, 1], [-9, 16, 0, 6, -1], [7, -10, 2, -7, 4], [2, 12, -9, 6, -1], [-15, 3, -14, 7, 3], [-16, 14, -4, 16, 0], [27, 10, -10, -3, -5], [37, 12, -1, 3, 0], [-10, 10, 3, -5, -4], [24, -4, -10, -14, -7], [-13, -28, 3, -4, 2], [32, -12, -3, -16, -1], [-26, 6, -10, -6, 6], [-10, -10, 6, -10, -4], [15, -5, -6, -11, 0], [42, 8, 6, -3, 2], [-2, 5, 14, -10, -6], [5, 4, 6, 10, 0], [27, 16, 4, -7, 0], [-24, 2, 7, 12, 5], [12, 4, -12, 17, 1], [9, 2, 10, -13, -5], [-36, 7, -4, 8, -4], [-18, 0, 11, 8, 5], [-3, 10, -3, 13, -1], [3, 6, -3, -5, 4], [5, 0, 13, 7, 7], [6, -16, -2, 10, -4], [28, -7, -3, -8, -1], [20, -11, -1, 4, 9], [16, -12, 10, -4, -4], [8, 12, 2, 8, -8], [35, 12, 12, -11, 8], [-20, 2, -12, 10, 6], [6, -4, 5, -14, 1], [8, 6, -19, -10, -5], [-35, 0, -4, 7, -10], [2, -2, -2, -4, 1], [2, 8, -3, -4, -3], [4, 6, 17, -15, 1], [12, -6, 13, -14, -3], [17, 8, -3, 7, 6], [20, 12, -4, -14, 0], [7, 0, -6, -1, 0], [28, -2, 5, -33, -1], [22, 8, -6, 10, 0], [6, 10, -2, 2, 0], [-20, 4, -8, -9, 7], [22, -4, 5, 5, -3], [-28, 0, -6, -4, 2], [0, 16, 3, 6, 8], [-16, -2, -2, -8, 0], [36, -8, 2, -4, -3], [28, 12, 1, 6, -4], [-20, -10, 7, -2, 3], [8, 5, 1, 18, 4], [38, -14, 0, 0, -7], [-5, -16, -9, -7, 4], [-15, -5, 19, 5, 2], [4, 22, -5, -2, 5], [-5, 14, -4, 3, 8], [11, -6, 4, 15, 8], [16, 20, 6, -6, 2], [28, 4, 3, 4, 10], [13, 6, 3, -5, 1], [11, 4, 6, 5, 1], [-22, 12, -8, 17, 7], [-3, -18, 11, 11, 2], [22, 4, 2, -24, 2], [54, -4, 2, -2, 2], [-9, 9, 19, -7, 2], [7, -6, 5, 5, -6], [50, 0, -1, 4, -6], [13, -4, 4, -19, -3], [22, -6, 5, 8, 3], [51, 14, -15, 3, -4], [42, -2, 8, -22, 0], [0, 16, -10, 0, -7], [-42, 22, -1, 10, 8], [-6, 4, 0, 2, 12], [-37, -4, 6, 11, 6], [28, 6, 0, 8, 6]]
